@@ -268,6 +268,14 @@ srl_build_decoder_struct(pTHX_ HV *opt)
         if ( (svp = hv_fetchs(opt, "use_undef", 0)) && SvTRUE(*svp))
             SRL_DEC_SET_OPTION(dec,SRL_F_DECODER_USE_UNDEF);
 
+        /* check if they want us to set all SVs readonly. */
+        if ( (svp = hv_fetchs(opt, "set_readonly", 0)) && SvTRUE(*svp))
+            SRL_DEC_SET_OPTION(dec, SRL_F_DECODER_SET_READONLY);
+
+        /* check if they want us to set normal scalars readonly. */
+        if ( (svp = hv_fetchs(opt, "set_readonly_scalars", 0)) && SvTRUE(*svp))
+            SRL_DEC_SET_OPTION(dec, SRL_F_DECODER_SET_READONLY_SCALARS);
+
     }
 
     return dec;
@@ -722,6 +730,7 @@ SRL_STATIC_INLINE void
 srl_finalize_structure(pTHX_ srl_decoder_t *dec)
 {
     int nobless = SRL_DEC_HAVE_OPTION(dec, SRL_F_DECODER_NO_BLESS_OBJECTS);
+
     if (dec->weakref_av)
         av_clear(dec->weakref_av);
     if (dec->ref_stashes) {
@@ -754,8 +763,17 @@ srl_finalize_structure(pTHX_ srl_decoder_t *dec)
 #if USE_588_WORKAROUND
                         /* was blessed early, don't rebless */
 #else
-                        if (!nobless)
-                            sv_bless(obj, stash);
+                        if (!nobless) {
+                            if ( SRL_DEC_HAVE_OPTION(dec, SRL_F_DECODER_READONLY_FLAGS) && SvREADONLY(SvRV(obj))) {
+                                /* the referenced scalar was readonly, temporary 
+                                   set it rw to bless its reference */
+                                SvREADONLY_off(SvRV(obj));
+                                sv_bless(obj, stash);
+                                SvREADONLY_on(SvRV(obj));
+                            } else {
+                                sv_bless(obj, stash);
+                            }
+                        }
 #endif
                     } else {
                         PTABLE_iter_free(it);
@@ -1804,6 +1822,7 @@ srl_read_single_value(pTHX_ srl_decoder_t *dec, SV* into)
 {
     STRLEN len;
     U8 tag;
+    int is_ref = 0;
     if (++dec->recursion_depth > dec->max_recursion_depth) {
         SRL_ERRORf1("Reached recursion limit (%lu) during deserialization",
                 (unsigned long)dec->max_recursion_depth);
@@ -1837,39 +1856,41 @@ srl_read_single_value(pTHX_ srl_decoder_t *dec, SV* into)
     else
     if ( IS_SRL_HDR_HASHREF(tag) ) {
         srl_read_hash(aTHX_ dec, into, tag);
+        is_ref = 1;
     }
     else
     if ( IS_SRL_HDR_ARRAYREF(tag) ) {
         srl_read_array(aTHX_ dec, into, tag);
+        is_ref = 1;
     }
     else {
         switch (tag) {
-            case SRL_HDR_VARINT:        srl_read_varint(aTHX_ dec, into);           break;
-            case SRL_HDR_ZIGZAG:        srl_read_zigzag(aTHX_ dec, into);           break;
+            case SRL_HDR_VARINT:        srl_read_varint(aTHX_ dec, into);                 break;
+            case SRL_HDR_ZIGZAG:        srl_read_zigzag(aTHX_ dec, into);                 break;
 
-            case SRL_HDR_FLOAT:         srl_read_float(aTHX_ dec, into);            break;
-            case SRL_HDR_DOUBLE:        srl_read_double(aTHX_ dec, into);           break;
-            case SRL_HDR_LONG_DOUBLE:   srl_read_long_double(aTHX_ dec, into);      break;
+            case SRL_HDR_FLOAT:         srl_read_float(aTHX_ dec, into);                  break;
+            case SRL_HDR_DOUBLE:        srl_read_double(aTHX_ dec, into);                 break;
+            case SRL_HDR_LONG_DOUBLE:   srl_read_long_double(aTHX_ dec, into);            break;
 
-            case SRL_HDR_TRUE:          sv_setsv(into, &PL_sv_yes);                 break;
-            case SRL_HDR_FALSE:         sv_setsv(into, &PL_sv_no);                  break;
+            case SRL_HDR_TRUE:          sv_setsv(into, &PL_sv_yes);                       break;
+            case SRL_HDR_FALSE:         sv_setsv(into, &PL_sv_no);                        break;
             case SRL_HDR_CANONICAL_UNDEF: /* fallthrough */
-            case SRL_HDR_UNDEF:         sv_setsv(into, &PL_sv_undef);               break;
-            case SRL_HDR_BINARY:        srl_read_string(aTHX_ dec, 0, into);        break;
-            case SRL_HDR_STR_UTF8:      srl_read_string(aTHX_ dec, 1, into);        break;
+            case SRL_HDR_UNDEF:         sv_setsv(into, &PL_sv_undef);                     break;
+            case SRL_HDR_BINARY:        srl_read_string(aTHX_ dec, 0, into);              break;
+            case SRL_HDR_STR_UTF8:      srl_read_string(aTHX_ dec, 1, into);              break;
 
-            case SRL_HDR_WEAKEN:        srl_read_weaken(aTHX_ dec, into);           break;
-            case SRL_HDR_REFN:          srl_read_refn(aTHX_ dec, into);             break;
-            case SRL_HDR_REFP:          srl_read_refp(aTHX_ dec, into);             break;
+            case SRL_HDR_WEAKEN:        srl_read_weaken(aTHX_ dec, into);       is_ref=1; break;
+            case SRL_HDR_REFN:          srl_read_refn(aTHX_ dec, into);         is_ref=1; break;
+            case SRL_HDR_REFP:          srl_read_refp(aTHX_ dec, into);         is_ref=1; break;
             case SRL_HDR_OBJECT_FREEZE:
-            case SRL_HDR_OBJECT:        srl_read_object(aTHX_ dec, into, tag);      break;
+            case SRL_HDR_OBJECT:        srl_read_object(aTHX_ dec, into, tag);  is_ref=1; break;
             case SRL_HDR_OBJECTV_FREEZE:
-            case SRL_HDR_OBJECTV:       srl_read_objectv(aTHX_ dec, into, tag);     break;
-            case SRL_HDR_COPY:          srl_read_copy(aTHX_ dec, into);             break;
-            case SRL_HDR_EXTEND:        srl_read_extend(aTHX_ dec, into);           break;
-            case SRL_HDR_HASH:          srl_read_hash(aTHX_ dec, into, 0);          break;
-            case SRL_HDR_ARRAY:         srl_read_array(aTHX_ dec, into, 0);         break;
-            case SRL_HDR_REGEXP:        srl_read_regexp(aTHX_ dec, into);           break;
+            case SRL_HDR_OBJECTV:       srl_read_objectv(aTHX_ dec, into, tag); is_ref=1; break;
+            case SRL_HDR_COPY:          srl_read_copy(aTHX_ dec, into);                   break;
+            case SRL_HDR_EXTEND:        srl_read_extend(aTHX_ dec, into);                 break;
+            case SRL_HDR_HASH:          srl_read_hash(aTHX_ dec, into, 0);                break;
+            case SRL_HDR_ARRAY:         srl_read_array(aTHX_ dec, into, 0);               break;
+            case SRL_HDR_REGEXP:        srl_read_regexp(aTHX_ dec, into);                 break;
 
             case SRL_HDR_PAD:           /* no op */
                 while (BUF_NOT_DONE(dec) && *dec->pos == SRL_HDR_PAD)
@@ -1882,5 +1903,11 @@ srl_read_single_value(pTHX_ srl_decoder_t *dec, SV* into)
         }
     }
 
+    /* they want us to set all SVs readonly, or only the non-ref */
+    if (  SRL_DEC_HAVE_OPTION(dec, SRL_F_DECODER_SET_READONLY) ||
+          (SRL_DEC_HAVE_OPTION(dec, SRL_F_DECODER_SET_READONLY_SCALARS) && !is_ref) ) {
+        SvREADONLY_on(into);
+    }
+
     dec->recursion_depth--;
 }
